@@ -12,7 +12,7 @@ import subprocess
 if TYPE_CHECKING:
     from .instr import Instr
     from typing import *
-    from csoundlib import AudioDevice
+    from csoundlib import AudioDevice, MidiDevice
     
 
 _registry: Dict[str, Any] = {}
@@ -113,27 +113,27 @@ def instrResolveArgs(instr: Instr,
     return allargs
 
 
+def addNotifycationAtStop(body: str, notifyDeallocInstrnum: int) -> str:
+    notifystr = f'atstop {notifyDeallocInstrnum}, 0.01, 0.0, p1'
+    out = "\n".join([notifystr, body])
+    return out
+
+
 def instrWrapBody(body:str, instrid:Union[int, str, Sequence[str]], comment:str= '',
-                  addNotificationCode=False) -> str:
+                  notifyDeallocInstrnum: int = 0
+                  ) -> str:
     s = r"""
 instr {instrnum}  {commentstr}
-    {notifystr}
     {body}
 endin
     """
-    if addNotificationCode:
-        # notifystr = 'defer "outvalue", "__dealloc__", p1'
-        # TODO: defer can cause memory corruption in some cases
-        notifystr = 'atstop "_notifyDealloc", 0.01, 0.0, p1'
-    else:
-        notifystr = ''
+    if notifyDeallocInstrnum > 0:
+        body = addNotifycationAtStop(body, notifyDeallocInstrnum)
     commentstr = "; " + comment if comment else ""
     if isinstance(instrid, (list, tuple)):
         instrid = ", ".join([str(i) for i in instrid])
-    s = s.format(instrnum=instrid, body=body, notifystr=notifystr,
-                 commentstr=commentstr)
-    s = textwrap.dedent(s)
-    return s
+    s = s.format(instrnum=instrid, body=body, commentstr=commentstr)
+    return textwrap.dedent(s)
 
 
 def addLineNumbers(code: str) -> str:
@@ -210,6 +210,32 @@ def selectAudioDevice(devices: List[AudioDevice], title='Select device'
     return outdev
 
 
+def selectMidiDevice(devices: List[MidiDevice], title='Select MIDI device'
+                     ) -> Optional[MidiDevice]:
+    """
+    Select a midi device from the given devices
+
+    Args:
+        devices: the midi devices to select from, as returned from ...
+        title: the title of the dialog
+
+    Returns:
+        the deviceid of the selected device, None if no selection was made
+        If the devices are input devices, 'all' is added as option. The given
+        value can be passed to -M csound option
+    """
+    if len(devices) == 1:
+        return devices[0]
+    names = [f"{dev.name} [{dev.deviceid}]" for dev in devices]
+    selected = emlib.dialogs.selectItem(items=names, title=title)
+    if not selected:
+        return None
+    else:
+        name, devid = selected[:-1].split("[")
+        return next(d for d in devices if d.deviceid == devid)
+
+
+
 def selectItem(items: List[str], title="Select") -> Optional[str]:
     return emlib.dialogs.selectItem(items=items, title=title)
 
